from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel


class TmsTarif(TmsExtendedModel):
    _path_url = "/tarifs/"

    def __init__(self, tarif_name: str, tarif_tag: int, enabled: bool, provider: int = None, id: int = None):

        self.id = id
        self.tarif_name = tarif_name
        self.provider = provider
        self.tarif_tag = tarif_tag
        self.enabled = enabled

    @staticmethod
    def _dict_to_object(tarif_dict: dict) -> object:
        tarif = TmsTarif(
            id=tarif_dict["id"],
            tarif_name=tarif_dict["tarif_name"],
            provider=tarif_dict["provider"],
            tarif_tag=tarif_dict["tarif_tag"],
            enabled=tarif_dict["enabled"]
        )

        return tarif

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", channel: int = None, enabled: bool = None,
                 provider: int = None, tarif_tag: int = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:

        tarifs = super().get_list(start=start, limit=limit, sort=sort, channel=channel, enabled=enabled,
                                  provider=provider, tarif_tag=tarif_tag, quick_search=quick_search)

        return tarifs

    def __str__(self):
        return """id: {}, tarif_name:{}, provider:{}, tarif_tag:{}, enabled:{}""".format(
            self.id,
            self.tarif_name,
            self.provider,
            self.tarif_name,
            self.enabled
        )



import json
import requests
from tmsproviderapisdk.tms_config import TmsConfigHolder
from tmsproviderapisdk.tms_exceptions import ApiHTTPError, IdEmptyError, NotJsonDataError
from tmsproviderapisdk.tms_base_model import TmsBaseModel


class TmsExtendedModel(TmsBaseModel):

    def create(self) -> object:
        json_data = json.dumps(self.__dict__)

        try:
            r = requests.post(TmsConfigHolder.config().base_url + self._path_url,
                              headers=TmsConfigHolder.config().headers,
                              data=json_data)
        except Exception as e:
            print(e)
            return None

        if r.status_code != 200:
            raise ApiHTTPError("Create {}".format(self.__class__.__name__), r.status_code, r.text)

        try:
            resp = json.loads(r.text)
        except Exception:
            raise NotJsonDataError("Received non json data")

        o = self._dict_to_object(resp)

        return o

    def update(self) -> object:
        if self.id is None:
            raise IdEmptyError("{} Id cannot be empty".format(self.__class__.__name__))

        model = json.dumps(self.__dict__)

        try:
            r = requests.put(TmsConfigHolder.config().base_url + self._path_url + str(self.id),
                             headers=TmsConfigHolder.config().headers, data=model)
        except Exception as e:
            print(e)
            return None

        if r.status_code != 200:
            raise ApiHTTPError("Update {}".format(self.__class__.__name__), r.status_code, r.text)

        try:
            resp = json.loads(r.text)
        except Exception:
            raise NotJsonDataError("Received non json data")

        o = self._dict_to_object(resp)

        return o

    def delete(self) -> int:
        if self.id is None:
            raise IdEmptyError("{} Id cannot be empty".format(self.__class__.__name__))

        try:
            r = requests.delete(TmsConfigHolder.config().base_url + self._path_url + str(self.id),
                                headers=TmsConfigHolder.config().headers)
        except Exception as e:
            print(e)
            return None

        if r.status_code != 200:
            raise ApiHTTPError("Delete {}".format(self.__class__.__name__), r.status_code, r.text)

        return r.status_code



from typing import Optional, List, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel


class TmsAccountSubscription(TmsExtendedModel):
    _path_url = "/account_subscriptions/"

    def __init__(self, account: int, start: str, tarif: int, id: int = None, stop: str = None):
        self.account = account
        self.start = start
        self.tarif = tarif
        self.id = id
        self.stop = stop

    @staticmethod
    def _dict_to_object(as_dict: dict) -> object:
        account_subscription = TmsAccountSubscription(
            account=as_dict["account"],
            id=as_dict["id"],
            start=as_dict["start"],
            stop=as_dict["stop"],
            tarif=as_dict["tarif"]
        )
        return account_subscription

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", account: int = None, tarif: int = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:
        account_subscriptions = super().get_list(start=start, limit=limit, sort=sort, account=account,
                                                 tarif=tarif, quick_search=quick_search)

        return account_subscriptions

    def __str__(self):
        return """id:{}, account:{}, start:{}, stop:{}, tarif:{}""".format(
            self.id,
            self.account,
            self.start,
            self.stop,
            self.tarif
        )



from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel


class TmsDeviceSubscription(TmsExtendedModel):
    _path_url = "/device_subscriptions/"

    def __init__(self, device: int, start: str, tarif: int, id: int = None, stop: str = None):

        self.device = device
        self.start = start
        self.tarif = tarif
        self.id = id
        self.stop = stop

    @staticmethod
    def _dict_to_object(as_dict: dict) -> object:
        device_subscription = TmsDeviceSubscription(
            device=as_dict["device"],
            id=as_dict["id"],
            start=as_dict["start"],
            stop=as_dict["stop"],
            tarif=as_dict["tarif"]
        )
        return device_subscription

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", device: int = None, tarif: int = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:

        device_subscriptions = super().get_list(start=start, limit=limit, sort=sort, device=device, tarif=tarif,
                                                quick_search=quick_search)

        return device_subscriptions

    def __str__(self):
        return """id:{}, device:{}, start:{}, stop:{}, tarif:{}""".format(
            self.id,
            self.device,
            self.start,
            self.stop,
            self.tarif
        )



from typing import Optional, List, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel
import hashlib


class TmsAccount(TmsExtendedModel):
    _path_url = '/accounts/'

    def __init__(self, login: str, fullname: str, account_desc: str = "", contract_info: str = "",
                 devices_per_account_limit: int = None, enabled: bool = True, main_address: str = "", pin_md5: str = "",
                 remote_custom_field: str = "", provider: int = None, id: int = None):

        self.login = login
        self.fullname = fullname
        self.account_desc = account_desc
        self.contract_info = contract_info
        self.devices_per_account_limit = devices_per_account_limit
        self.enabled = enabled
        self.main_address = main_address
        self.pin_md5 = pin_md5
        self.remote_custom_field = remote_custom_field
        self.provider = provider
        self.id = id

    @staticmethod
    def get_md5_pin(clear_text_password: str) -> str:
        hash_password = hashlib.md5(clear_text_password.encode())
        pin_md5 = hash_password.hexdigest()

        return pin_md5

    @staticmethod
    def _dict_to_object(account_dict: dict) -> object:
        a: TmsAccount = TmsAccount(login=account_dict["login"],
                                   account_desc=account_dict["account_desc"],
                                   contract_info=account_dict["contract_info"],
                                   devices_per_account_limit=account_dict["devices_per_account_limit"],
                                   enabled=account_dict["enabled"],
                                   fullname=account_dict["fullname"],
                                   main_address=account_dict["main_address"],
                                   pin_md5=account_dict["pin_md5"],
                                   remote_custom_field=account_dict["remote_custom_field"],
                                   provider=account_dict["provider"],
                                   id=account_dict["id"])

        return a

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", provider: int = None, enabled: bool = None,
                 login: int = None, remote_custom_field: str = "",
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:

        accounts: Optional[Tuple[List[object], int]] = super().get_list(start, limit, sort=sort, provider=provider,
                                                                        enabled=enabled, login=login,
                                                                        remote_custom_field=remote_custom_field,
                                                                        quick_search=quick_search)

        return accounts

    def __str__(self):
        return """id:{}, login:{}, fullname:{}, remote_custom_field:{}, pin_md5:{}, contract_info:{},\
main_address:{}, account_desc:{}, provider:{}, enabled:{}, devices_per_account_limit:{}""".format(
            self.id,
            self.login,
            self.fullname,
            self.remote_custom_field,
            self.pin_md5,
            self.contract_info,
            self.main_address,
            self.account_desc,
            self.provider,
            self.enabled,
            self.devices_per_account_limit
        )



from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_base_model import TmsBaseModel


class TmsProvider(TmsBaseModel):

    _path_url = "/providers/"

    def __init__(self, id: int = None, region_tag: int = None, provider_name: str = None, provider_comment: str = None,
                 enabled: bool = False, logo_url: str = None, devices_per_account_limit: int = None):

        self.id = id
        self.region_tag = region_tag
        self.provider_name = provider_name
        self.provider_comment = provider_comment
        self.enabled = enabled
        self.logo_url = logo_url
        self.devices_per_account_limit = devices_per_account_limit

    @staticmethod
    def _dict_to_object(provider_dict: dict) -> object:
        p = TmsProvider(
            id=provider_dict["id"],
            region_tag=provider_dict["region_tag"],
            provider_name=provider_dict["provider_name"],
            provider_comment=provider_dict["provider_comment"],
            enabled=provider_dict["enabled"],
            logo_url=provider_dict["logo_url"],
            devices_per_account_limit=provider_dict["devices_per_account_limit"]
        )

        return p

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", enabled: bool = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:
        providers: Optional[Tuple[List[object], int]] = super().get_list(start=start, limit=limit, sort=sort,
                                                                         enabled=enabled, quick_search=quick_search)

        return providers

    def __str__(self):
        return """id: {}, region_tag: {}, provider_name: {}, provider_comment:{}, enabled: {},\
logo_url: {}, devices_per_account_limit: {}""".format(
            self.id,
            self.region_tag,
            self.provider_name,
            self.provider_comment,
            self.enabled,
            self.logo_url,
            self.devices_per_account_limit
        )



from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_base_model import TmsBaseModel


class TmsChannel(TmsBaseModel):
    _path_url = "/channels/"

    def __init__(self, id: int, name: str = "", text_name: str = "", display_number: str = "", logo_url: str = "",
                 enabled: bool = False, time_shift_depth: int = None):

        self.id = id
        self.name = name
        self.text_name = text_name
        self.display_number = display_number
        self.logo_url = logo_url
        self.enabled = enabled
        self.time_shift_depth = time_shift_depth

    @staticmethod
    def _dict_to_object(channel_dict: dict) -> object:
        channel = TmsChannel(
            id=channel_dict["id"],
            name=channel_dict["name"],
            text_name=channel_dict["text_name"],
            display_number=channel_dict["display_number"],
            logo_url=channel_dict["logo_url"],
            enabled=channel_dict["enabled"],
            time_shift_depth=channel_dict["time_shift_depth"]
        )
        return channel

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", tarif: int = None, enabled: bool = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:
        channels = super().get_list(start=start, limit=limit, sort=sort, tarif=tarif, quick_search=quick_search)

        return channels

    def __str__(self):
        return """id:{}, name:{}, text_name:{}, display_number:{}, logo_url:{}, \
            enabled: {}, time_shift_depth: {}""".format(
                self.id,
                self.name,
                self.text_name,
                self.display_number,
                self.logo_url,
                self.enabled,
                self.time_shift_depth
            )



from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel


class TmsTarifTag(TmsExtendedModel):
    _path_url = "/tarif_tags/"

    def __init__(self, enabled: bool, name: str, provider: int, id: int = None, position: int = None):

        self.enabled = enabled
        self.name = name
        self.provider = provider
        self.id = id
        self.position = position

    @staticmethod
    def _dict_to_object(tarif_tag_dict: dict) -> object:
        tarif_tag = TmsTarifTag(
            id=tarif_tag_dict["id"],
            name=tarif_tag_dict["name"],
            enabled=tarif_tag_dict["enabled"],
            provider=tarif_tag_dict["provider"],
            position=tarif_tag_dict["position"]
        )

        return tarif_tag

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, sort: str = "", enabled: bool = None, provider: int = None,
                 quick_search: str = "") -> Optional[Tuple[List[object], int]]:

        tarif_tags = super().get_list(start=start, limit=limit, sort=sort, enabled=enabled, provider=provider,
                                      quick_search=quick_search)

        return tarif_tags

    def __str__(self):
        return """id: {}, name: {}, enabled: {}, provider: {}, position: {}""".format(
            self.id,
            self.name,
            self.enabled,
            self.provider,
            self.position
        )



from tmsproviderapisdk.tms_config import TmsConfig, TmsConfigHolder
from tmsproviderapisdk.tms_account import TmsAccount
from tmsproviderapisdk.tms_account_subscription import TmsAccountSubscription
from tmsproviderapisdk.tms_channel import TmsChannel
from tmsproviderapisdk.tms_device import TmsDevice
from tmsproviderapisdk.tms_device_subscription import TmsDeviceSubscription
from tmsproviderapisdk.tms_tarif import TmsTarif
from tmsproviderapisdk.tms_tarif_tag import TmsTarifTag



import json
import requests
from tmsproviderapisdk.tms_config import TmsConfigHolder
from tmsproviderapisdk.tms_exceptions import ApiHTTPError, NotJsonDataError
from typing import List, Optional, Tuple


class TmsBaseModel:
    _path_url: str

    @classmethod
    def get(cls, id):
        """

        :param id: int
        :return: object
        """
        try:
            r = requests.get(TmsConfigHolder.config().base_url + cls._path_url + str(id),
                             headers=TmsConfigHolder.config().headers)
        except Exception as e:
            print(e)
            return None

        if r.status_code != 200:
            raise ApiHTTPError("Get {}".format(cls.__name__), r.status_code, r.text)

        try:
            resp = json.loads(r.text)
        except Exception:
            raise NotJsonDataError("Received non json data")

        o = cls._dict_to_object(resp)

        return o

    @classmethod
    def get_list(cls, start: int = 0, limit: int = 50, **kwargs: any) -> Optional[Tuple[List[object], int]]:
        objects = []

        query = "?start={}&limit={}".format(start, limit)

        for name, value in kwargs.items():
            if value != '' and value is not None:
                query += "&{}={}".format(name, value)

        try:
            r = requests.get(
                TmsConfigHolder.config().base_url + cls._path_url + query,
                headers=TmsConfigHolder.config().headers)
        except Exception as e:
            print(e)
            return None

        if r.status_code != 200:
            raise ApiHTTPError("Get {}".format(cls.__name__), r.status_code, r.text)

        try:
            resp = json.loads(r.text)
        except Exception:
            raise NotJsonDataError("Received non json data")

        if not resp.get("data"):
            print("Response not have data, response: {}".format(r.text))
            return None

        for ra in resp["data"]:
            o = cls._dict_to_object(ra)
            objects.append(o)

        total_count = resp["total"]

        return objects, total_count

    @staticmethod
    def _dict_to_object(dict_model: dict) -> object:
        pass



from typing import List, Optional, Tuple
from tmsproviderapisdk.tms_extended_model import TmsExtendedModel


class TmsDevice(TmsExtendedModel):
    _path_url = "/devices/"

    def __init__(self, unique_id: str, device_id: int = None, ipaddr: str = None, mac: str = None,
                 remote_custom_field: str = None, comment: str = None, last_online: str = None, last_fw_ver: str = None,
                 first_online: str = None, use_nat: bool = False, operation_system: str = None, udpxy_addr: str = None,
                 device_type: int = None, provider: int = None, account: int = None):

        self.unique_id = unique_id
        self.id = device_id
        self.ipaddr = ipaddr
        self.mac = mac
        self.remote_custom_field = remote_custom_field
        self.comment = comment
        self.last_online = last_online
        self.last_fw_ver = last_fw_ver
        self.first_online = first_online
        self.use_nat = use_nat
        self.operation_system = operation_system
        self.udpxy_addr = udpxy_addr
        self.device_type = device_type
        self.provider = provider
        self.account = account

    @staticmethod
    def _dict_to_object(device_dict: dict) -> object:
        device = TmsDevice(
            unique_id=device_dict["unique_id"],
            device_id=device_dict["id"],
            ipaddr=device_dict["ipaddr"],
            mac=device_dict["mac"],
            remote_custom_field=device_dict["remote_custom_field"],
            comment=device_dict["comment"],
            last_online=device_dict["last_online"],
            last_fw_ver=device_dict["last_fw_ver"],
            first_online=device_dict["first_online"],
            use_nat=device_dict["use_nat"],
            operation_system=device_dict["operation_system"],
            udpxy_addr=device_dict["udpxy_addr"],
            device_type=device_dict["device_type"],
            provider=device_dict["provider"],
            account=device_dict["account"]
        )

        return device

    @classmethod
    def get_list(cls, account: int = None, device_type: int = None, limit: int = 50, provider: int = None,
                 quick_search: str = "", remote_custom_field: str = None, sort: str = "", start: int = 0,
                 unique_id: str = "") -> Optional[Tuple[List[object], int]]:

        devices = super().get_list(start=start, limit=limit, account=account, device_type=device_type,
                                   provider=provider, quick_search=quick_search,
                                   remote_custom_field=remote_custom_field, sort=sort, unique_id=unique_id)

        return devices

    def __str__(self):
        return """id:{}, ipaddr:{}, mac:{}, unique_id:{}, remote_custom_field: {}, comment: {}, last_online: {}, \
        last_fw_ver: {}, first_online: {}, use_nat: {}, operation_system: {}, \
        udpxy_addr: {}, device_type: {}, provider: {}, account: {}""".format(
            self.id,
            self.ipaddr,
            self.mac,
            self.unique_id,
            self.remote_custom_field,
            self.comment,
            self.last_online,
            self.last_fw_ver,
            self.first_online,
            self.use_nat,
            self.operation_system,
            self.udpxy_addr,
            self.device_type,
            self.provider,
            self.account
        )



import json


class Error(Exception):
    pass


class ApiHTTPError(Error):
    def __init__(self, context, status_code, response_text):
        self.context = context
        self.status_code = status_code
        self.response_text = response_text

    def __str__(self):
        message = "Api HTTP Error on {}, http status code: {}".format(
            self.context,
            self.status_code
        )

        if 399 < self.status_code < 500:
            text = json.loads(self.response_text)
            cause = text.get("cause")
            if not cause:
                cause = text.get("error")
            status_code = text.get("statusCode", self.status_code)

            message = "Api HTTP Error on {}, status code: {}, cause: {}".format(
                self.context,
                status_code,
                cause
            )

        return message


class IdEmptyError(Error):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class ProviderEmptyError(Error):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class DeviceIdEmptyError(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class SubsriptionIdEmptyError(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class TarifIdEmptyError(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class TarifTagIdEmptyError(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class NotJsonDataError(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message



import base64


class TmsConfig:

    def __init__(self, base_url: str, username: str = None, password: str = None, token: str = None):

        self.base_url = base_url
        self.username = username
        self.password = password
        self.token = token
        self.headers = None
        self._configure()

    def _configure(self):
        self.base_url = self.base_url + '/api/provider'

        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        if self.username and (self.password or self.token):
            if self.password:
                login_password = "{}:{}".format(
                    self.username,
                    self.password
                ).encode()
            if self.token:
                login_password = "{}:{}".format(
                    self.username,
                    self.token
                ).encode()

            key = base64.b64encode(bytes(login_password))
            headers.update({'Authorization': "Basic {}".format(key.decode())})

        self.headers = headers


class TmsConfigHolder:
    _config: object

    @staticmethod
    def config():
        return TmsConfigHolder._config

    @staticmethod
    def apply(config: object):
        TmsConfigHolder._config = config
